import { getModelToken } from '@nestjs/mongoose';
import { Test, TestingModule } from '@nestjs/testing';
import { appointmentMockData, singleAppointmentMock } from '../../test/mock/data/appointment.mock.data';
import { AppointmentController } from './appointment.controller';
import { Appointment } from './appointment.schema';
import { AppointmentService } from './appointment.service';

describe('UserRegistryController', () => {
  let controller: AppointmentController;

  const appointmentServiceMock = {
    findAll: jest.fn(),
    create: jest.fn(),
  };

  beforeEach(async () => {
    const module: TestingModule = await Test.createTestingModule({
      imports: [],
      providers: [
        {
          provide: AppointmentService,
          useValue: appointmentServiceMock,
        },
        {
          provide: getModelToken('Appointment'),
          useValue: Appointment,
        },
      ],
      controllers: [AppointmentController],
    }).compile();

    controller = module.get<AppointmentController>(AppointmentController);
  });

  it('should be defined', () => {
    expect(controller).toBeDefined();
  });

  describe('findAll', () => {
    it('should findAll entries', async () => {
      appointmentServiceMock.findAll.mockResolvedValue(appointmentMockData);
      const reply = await controller.findAll();
      expect(reply).toStrictEqual(appointmentMockData);
    });
  });
  describe('createAppointment', () => {
    it('should create a new entry', async () => {
      appointmentServiceMock.create.mockResolvedValue(singleAppointmentMock);
      const reply = await controller.createAppointment(singleAppointmentMock);
      expect(reply).toStrictEqual(singleAppointmentMock);
    });
  });
});
