import { getModelToken } from '@nestjs/mongoose';
import { Test, TestingModule } from '@nestjs/testing';
import { AppointmentService } from './appointment.service';
import { appointmentMockData } from '../../test/mock/data/appointment.mock.data';
import { MailerService } from '../mailer/mailer.service';
import { MailerModule } from '../mailer/mailer.module';
import { HttpModule } from '@nestjs/axios';
import { UsersService } from '../users/services/users.service';
import { StructuresService } from '../structures/services/structures.service';
import { UsersServiceMock } from '../../test/mock/services/user.mock.service';
import { StructuresServiceMock } from '../../test/mock/services/structures.mock.service';
import * as ejs from 'ejs';

jest.mock('ejs');
describe('userRegistryService', () => {
  let service: AppointmentService;
  const mockAppointmentModel = {
    find: jest.fn(() => mockAppointmentModel),
    create: jest.fn(() => mockAppointmentModel),
    sort: jest.fn(() => mockAppointmentModel),
    exec: jest.fn(() => mockAppointmentModel),
    deleteMany: jest.fn(() => mockAppointmentModel),
  };
  const mockMailService = {
    send: jest.fn(),
    getTemplateLocation: jest.fn(),
    loadJsonConfig: jest.fn(),
    config: {
      templates: {
        appointment: {
          ejs: 'appointment.ejs',
          json: 'appointment.json',
        },
      },
    },
  };

  const ejsSpy = jest.spyOn(ejs, 'renderFile');
  const mailerSpy = jest.spyOn(mockMailService, 'send');

  beforeEach(async () => {
    const module: TestingModule = await Test.createTestingModule({
      imports: [MailerModule, HttpModule],
      providers: [
        AppointmentService,
        {
          provide: UsersService,
          useClass: UsersServiceMock,
        },
        {
          provide: StructuresService,
          useClass: StructuresServiceMock,
        },
        { provide: MailerService, useValue: mockMailService },
        {
          provide: getModelToken('Appointment'),
          useValue: mockAppointmentModel,
        },
      ],
    }).compile();
    service = module.get<AppointmentService>(AppointmentService);
  });

  it('should be defined', () => {
    expect(service).toBeDefined();
  });

  describe('findAll', () => {
    it('should findAll entries', async () => {
      mockAppointmentModel.exec.mockResolvedValueOnce(appointmentMockData);
      expect(await service.findAll()).toBe(appointmentMockData);
    });
  });
  describe('create', () => {
    it('should create an entry', async () => {
      const res = {
        accompanimentType: 'RDV Conseiller Numérique',
        name: 'test',
        onlineDemarcheType: ['Démarches Métropolitaines', 'CPAM'],
        phone: '65 65',
        surname: 'teztrfzegfv',
        structureOrientator: { structureName: 'name', structureMail: 'mail@structure.org', structurePhone: '12 34' },
        structureRDV: '6093ba0e2ab5775cfc01ed3e',
        socialWorkerId: '60648b104067f10013d69929',
      };
      mockMailService.loadJsonConfig.mockReturnValueOnce({ subject: 'Teste Mail' });
      ejsSpy.mockResolvedValueOnce('coucou');
      mockAppointmentModel.create.mockResolvedValueOnce(res);
      expect(await service.create(res)).toStrictEqual(res);
      expect(mailerSpy).toHaveBeenCalledTimes(1);
    });
  });
});
