import { HttpModule } from '@nestjs/axios';
import { getModelToken } from '@nestjs/mongoose';
import { Test, TestingModule } from '@nestjs/testing';
import { CreateCategories } from '../dto/create-categories.dto';
import { CategoriesService } from './categories.service';

describe('CategoriesService', () => {
  let service: CategoriesService;

  const mockCategoriesModel = {
    create: jest.fn(),
    find: jest.fn(),
    findOne: jest.fn(),
    select: jest.fn(),
    exec: jest.fn(),
  };

  beforeEach(async () => {
    const module: TestingModule = await Test.createTestingModule({
      imports: [HttpModule],
      providers: [
        CategoriesService,
        {
          provide: getModelToken('Categories'),
          useValue: mockCategoriesModel,
        },
      ],
    }).compile();

    service = module.get<CategoriesService>(CategoriesService);
  });

  it('should be defined', () => {
    expect(service).toBeDefined();
  });

  it('should findAll structures', async () => {
    const data: CreateCategories[] = [
      { id: 'categoryId', theme: 'Public', name: 'test', modules: [{ id: 'Test', name: 'Text du test' }] },
      { id: 'categoryId', theme: 'Public', name: 'test2', modules: [{ id: 'Test2', name: 'Text du test test 2' }] },
    ];
    mockCategoriesModel.find.mockReturnThis();
    mockCategoriesModel.exec.mockResolvedValueOnce(data);
    expect(await service.findAll()).toEqual(data);
  });
  it('should findOneComplete structures', async () => {
    const data: CreateCategories[] = [
      { id: 'categoryId', theme: 'Public', name: 'test', modules: [{ id: 'Test', name: 'Text du test' }] },
      { id: 'categoryId', theme: 'Public', name: 'test2', modules: [{ id: 'Test2', name: 'Text du test test 2' }] },
    ];
    mockCategoriesModel.findOne.mockReturnThis();
    mockCategoriesModel.exec.mockResolvedValueOnce(data);
    expect(await service.findOneComplete('categoryId')).toEqual(data);
  });
});
