import { getDb } from '../migrations-utils/db';

class ApticModule {
  id: string;
  display_id: string;
  display_name: string;
  url: string;
  last_update_time: string;
  receipt_time: string;
}

class ApticDoc {
  id: string;
  modules: ApticModule[];
}

export const up = async () => {
  const db = await getDb();
  await updateStructuresId(db);
  await updateApticReferential(db);
};

export const down = async () => {
  const db = await getDb();
  await downgradeApticReferential(db);
  await downgradeStructuresId(db);
};

async function downgradeStructuresId(db) {
  await db
    .collection('structures')
    .find({})
    .forEach((doc) => {
      const newDoc = downgradeStructure(doc);
      db.collection('structures').updateMany({ _id: doc._id }, [{ $set: newDoc }]);
    });
}

async function updateStructuresId(db) {
  await db
    .collection('structures')
    .find({})
    .forEach((doc) => {
      const newDoc = updateStructure(doc);
      db.collection('structures').updateMany({ _id: doc._id }, [{ $set: newDoc }]);
    });
}

function updateStructure(structure): any {
  let newArray = [];
  // Social and professional
  newArray.push(switchStructureId(structure.socialAndProfessional, '254', '6'));
  newArray.push(switchStructureId(structure.socialAndProfessional, '240', '20'));
  newArray.push(switchStructureId(structure.socialAndProfessional, '194', '66'));
  newArray.push(switchStructureId(structure.socialAndProfessional, '193', '67'));
  newArray.push(switchStructureId(structure.socialAndProfessional, '192', '68'));
  newArray.push(switchStructureId(structure.socialAndProfessional, '191', '69'));
  newArray.push(switchStructureId(structure.socialAndProfessional, '262', '124'));
  newArray.push(switchStructureId(structure.socialAndProfessional, '263', '125'));
  newArray.push(switchStructureId(structure.socialAndProfessional, '003', '127'));
  // Remove null cases
  newArray = newArray.filter((obj) => obj);
  structure.socialAndProfessional = newArray;

  // Base Skills
  newArray = [];
  newArray.push(switchStructureId(structure.baseSkills, '260', '260'));
  newArray.push(switchStructureId(structure.baseSkills, '259', '1'));
  newArray.push(switchStructureId(structure.baseSkills, '261', '11'));
  newArray.push(switchStructureId(structure.baseSkills, '222', '38'));
  newArray.push(switchStructureId(structure.baseSkills, '212', '48'));
  newArray.push(switchStructureId(structure.baseSkills, '186', '74'));
  newArray.push(switchStructureId(structure.baseSkills, '183', '77'));
  // Remove null cases
  newArray = newArray.filter((obj) => obj);
  structure.baseSkills = newArray;

  // Access Right
  newArray = [];
  newArray.push(switchStructureId(structure.accessRight, '176', '84'));
  newArray.push(switchStructureId(structure.accessRight, '175', '85'));
  newArray.push(switchStructureId(structure.accessRight, '174', '86'));
  newArray.push(switchStructureId(structure.accessRight, '173', '87'));
  newArray.push(switchStructureId(structure.accessRight, '172', '88'));
  newArray.push(switchStructureId(structure.accessRight, '171', '89'));
  newArray.push(switchStructureId(structure.accessRight, '167', '93'));
  newArray.push(switchStructureId(structure.accessRight, '165', '95'));
  // Remove null cases
  newArray = newArray.filter((obj) => obj);
  structure.accessRight = newArray;

  // Parenting
  newArray = [];
  newArray.push(switchStructureId(structure.parentingHelp, '257', '3'));
  newArray.push(switchStructureId(structure.parentingHelp, '238', '22'));
  newArray.push(switchStructureId(structure.parentingHelp, '178', '82'));
  newArray.push(switchStructureId(structure.parentingHelp, '166', '94'));
  // Remove null cases
  newArray = newArray.filter((obj) => obj);
  structure.parentingHelp = newArray;

  // Digital Security
  newArray = [];
  newArray.push(switchStructureId(structure.digitalCultureSecurity, '264', '2'));
  newArray.push(switchStructureId(structure.digitalCultureSecurity, '255', '5'));
  newArray.push(switchStructureId(structure.digitalCultureSecurity, '265', '9'));
  newArray.push(switchStructureId(structure.digitalCultureSecurity, '232', '28'));
  newArray.push(switchStructureId(structure.digitalCultureSecurity, '225', '34'));
  newArray.push(switchStructureId(structure.digitalCultureSecurity, '221', '39'));
  newArray.push(switchStructureId(structure.digitalCultureSecurity, '218', '42'));
  newArray.push(switchStructureId(structure.digitalCultureSecurity, '209', '51'));
  newArray.push(switchStructureId(structure.digitalCultureSecurity, '208', '52'));
  newArray.push(switchStructureId(structure.digitalCultureSecurity, '206', '54'));
  newArray.push(switchStructureId(structure.digitalCultureSecurity, '195', '65'));
  newArray.push(switchStructureId(structure.digitalCultureSecurity, '164', '96'));
  newArray.push(switchStructureId(structure.digitalCultureSecurity, '163', '97'));
  newArray.push(switchStructureId(structure.digitalCultureSecurity, '162', '98'));
  // Remove null cases
  newArray = newArray.filter((obj) => obj);
  structure.digitalCultureSecurity = newArray;

  return structure;
}

function downgradeStructure(structure): any {
  let newArray = [];
  // Social and professional
  newArray.push(switchStructureId(structure.socialAndProfessional, '6', '254'));
  newArray.push(switchStructureId(structure.socialAndProfessional, '20', '240'));
  newArray.push(switchStructureId(structure.socialAndProfessional, '66', '194'));
  newArray.push(switchStructureId(structure.socialAndProfessional, '67', '193'));
  newArray.push(switchStructureId(structure.socialAndProfessional, '68', '192'));
  newArray.push(switchStructureId(structure.socialAndProfessional, '69', '191'));
  newArray.push(switchStructureId(structure.socialAndProfessional, '124', '262'));
  newArray.push(switchStructureId(structure.socialAndProfessional, '125', '263'));
  newArray.push(switchStructureId(structure.socialAndProfessional, '127', '003'));
  // Remove null cases
  newArray = newArray.filter((obj) => obj);
  structure.socialAndProfessional = newArray;

  // Base Skills
  newArray = [];
  newArray.push(switchStructureId(structure.baseSkills, '260', '260'));
  newArray.push(switchStructureId(structure.baseSkills, '1', '259'));
  newArray.push(switchStructureId(structure.baseSkills, '11', '261'));
  newArray.push(switchStructureId(structure.baseSkills, '38', '222'));
  newArray.push(switchStructureId(structure.baseSkills, '48', '212'));
  newArray.push(switchStructureId(structure.baseSkills, '74', '186'));
  newArray.push(switchStructureId(structure.baseSkills, '77', '183'));
  // Remove null cases
  newArray = newArray.filter((obj) => obj);
  structure.baseSkills = newArray;

  // Access Right
  newArray = [];
  newArray.push(switchStructureId(structure.accessRight, '84', '176'));
  newArray.push(switchStructureId(structure.accessRight, '85', '175'));
  newArray.push(switchStructureId(structure.accessRight, '86', '174'));
  newArray.push(switchStructureId(structure.accessRight, '87', '173'));
  newArray.push(switchStructureId(structure.accessRight, '88', '172'));
  newArray.push(switchStructureId(structure.accessRight, '89', '171'));
  newArray.push(switchStructureId(structure.accessRight, '93', '167'));
  newArray.push(switchStructureId(structure.accessRight, '95', '165'));
  // Remove null cases
  newArray = newArray.filter((obj) => obj);
  structure.accessRight = newArray;

  // Parenting
  newArray = [];
  newArray.push(switchStructureId(structure.parentingHelp, '3', '257'));
  newArray.push(switchStructureId(structure.parentingHelp, '22', '238'));
  newArray.push(switchStructureId(structure.parentingHelp, '82', '178'));
  newArray.push(switchStructureId(structure.parentingHelp, '94', '166'));
  // Remove null cases
  newArray = newArray.filter((obj) => obj);
  structure.parentingHelp = newArray;

  // Digital Security
  newArray = [];
  newArray.push(switchStructureId(structure.digitalCultureSecurity, '2', '264'));
  newArray.push(switchStructureId(structure.digitalCultureSecurity, '5', '255'));
  newArray.push(switchStructureId(structure.digitalCultureSecurity, '9', '265'));
  newArray.push(switchStructureId(structure.digitalCultureSecurity, '28', '232'));
  newArray.push(switchStructureId(structure.digitalCultureSecurity, '34', '225'));
  newArray.push(switchStructureId(structure.digitalCultureSecurity, '39', '221'));
  newArray.push(switchStructureId(structure.digitalCultureSecurity, '42', '218'));
  newArray.push(switchStructureId(structure.digitalCultureSecurity, '51', '209'));
  newArray.push(switchStructureId(structure.digitalCultureSecurity, '52', '208'));
  newArray.push(switchStructureId(structure.digitalCultureSecurity, '54', '206'));
  newArray.push(switchStructureId(structure.digitalCultureSecurity, '65', '195'));
  newArray.push(switchStructureId(structure.digitalCultureSecurity, '96', '164'));
  newArray.push(switchStructureId(structure.digitalCultureSecurity, '97', '163'));
  newArray.push(switchStructureId(structure.digitalCultureSecurity, '98', '162'));
  // Remove null cases
  newArray = newArray.filter((obj) => obj);
  structure.digitalCultureSecurity = newArray;

  return structure;
}

async function updateApticReferential(db) {
  // Base Skills
  let newModule = { modules: [] };
  const baseSkills: ApticDoc = await db.collection('categoriesformations').findOne({ id: 'baseSkills' });
  newModule.modules.push(switchId(baseSkills.modules, '260', '260'));
  newModule.modules.push(switchId(baseSkills.modules, '259', '1'));
  newModule.modules.push(switchId(baseSkills.modules, '261', '11'));
  newModule.modules.push(switchId(baseSkills.modules, '222', '38'));
  newModule.modules.push(switchId(baseSkills.modules, '212', '48'));
  newModule.modules.push(switchId(baseSkills.modules, '186', '74'));
  newModule.modules.push(switchId(baseSkills.modules, '183', '77'));
  db.collection('categoriesformations').findOneAndUpdate(
    { id: 'baseSkills' },
    { $set: { modules: newModule.modules } }
  );

  // Access Right
  newModule = { modules: [] };
  const accessRight: ApticDoc = await db.collection('categoriesformations').findOne({ id: 'accessRight' });
  newModule.modules.push(switchId(accessRight.modules, '176', '84'));
  newModule.modules.push(switchId(accessRight.modules, '175', '85'));
  newModule.modules.push(switchId(accessRight.modules, '174', '86'));
  newModule.modules.push(switchId(accessRight.modules, '173', '87'));
  newModule.modules.push(switchId(accessRight.modules, '172', '88'));
  newModule.modules.push(switchId(accessRight.modules, '171', '89'));
  newModule.modules.push(switchId(accessRight.modules, '167', '93'));
  newModule.modules.push(switchId(accessRight.modules, '165', '95'));
  db.collection('categoriesformations').findOneAndUpdate(
    { id: 'accessRight' },
    { $set: { modules: newModule.modules } }
  );

  // Parenting
  newModule = { modules: [] };
  const parentingHelp: ApticDoc = await db.collection('categoriesformations').findOne({ id: 'parentingHelp' });
  newModule.modules.push(switchId(parentingHelp.modules, '257', '3'));
  newModule.modules.push(switchId(parentingHelp.modules, '238', '22'));
  newModule.modules.push(switchId(parentingHelp.modules, '178', '82'));
  newModule.modules.push(switchId(parentingHelp.modules, '166', '94'));
  db.collection('categoriesformations').findOneAndUpdate(
    { id: 'parentingHelp' },
    { $set: { modules: newModule.modules } }
  );

  // Social and professional
  newModule = { modules: [] };
  const socialAndProfessional: ApticDoc = await db
    .collection('categoriesformations')
    .findOne({ id: 'socialAndProfessional' });
  newModule.modules.push(switchId(socialAndProfessional.modules, '254', '6'));
  newModule.modules.push(switchId(socialAndProfessional.modules, '240', '20'));
  newModule.modules.push(switchId(socialAndProfessional.modules, '194', '66'));
  newModule.modules.push(switchId(socialAndProfessional.modules, '193', '67'));
  newModule.modules.push(switchId(socialAndProfessional.modules, '192', '68'));
  newModule.modules.push(switchId(socialAndProfessional.modules, '191', '69'));
  newModule.modules.push(switchId(socialAndProfessional.modules, '262', '124'));
  newModule.modules.push(switchId(socialAndProfessional.modules, '263', '125'));
  newModule.modules.push(switchId(socialAndProfessional.modules, '003', '127'));
  db.collection('categoriesformations').findOneAndUpdate(
    { id: 'socialAndProfessional' },
    { $set: { modules: newModule.modules } }
  );

  // Digital security
  newModule = { modules: [] };
  const digitalCultureSecurity: ApticDoc = await db
    .collection('categoriesformations')
    .findOne({ id: 'digitalCultureSecurity' });
  newModule.modules.push(switchId(digitalCultureSecurity.modules, '264', '2'));
  newModule.modules.push(switchId(digitalCultureSecurity.modules, '255', '5'));
  newModule.modules.push(switchId(digitalCultureSecurity.modules, '265', '9'));
  newModule.modules.push(switchId(digitalCultureSecurity.modules, '232', '28'));
  newModule.modules.push(switchId(digitalCultureSecurity.modules, '225', '34'));
  newModule.modules.push(switchId(digitalCultureSecurity.modules, '221', '39'));
  newModule.modules.push(switchId(digitalCultureSecurity.modules, '218', '42'));
  newModule.modules.push(switchId(digitalCultureSecurity.modules, '209', '51'));
  newModule.modules.push(switchId(digitalCultureSecurity.modules, '208', '52'));
  newModule.modules.push(switchId(digitalCultureSecurity.modules, '206', '54'));
  newModule.modules.push(switchId(digitalCultureSecurity.modules, '195', '65'));
  newModule.modules.push(switchId(digitalCultureSecurity.modules, '164', '96'));
  newModule.modules.push(switchId(digitalCultureSecurity.modules, '163', '97'));
  newModule.modules.push(switchId(digitalCultureSecurity.modules, '162', '98'));
  db.collection('categoriesformations').findOneAndUpdate(
    { id: 'digitalCultureSecurity' },
    { $set: { modules: newModule.modules } }
  );
}

async function downgradeApticReferential(db) {
  // Base Skills
  let newModule = { modules: [] };
  const baseSkills: ApticDoc = await db.collection('categoriesformations').findOne({ id: 'baseSkills' });
  newModule.modules.push(switchId(baseSkills.modules, '260', '260'));
  newModule.modules.push(switchId(baseSkills.modules, '1', '259'));
  newModule.modules.push(switchId(baseSkills.modules, '11', '261'));
  newModule.modules.push(switchId(baseSkills.modules, '38', '222'));
  newModule.modules.push(switchId(baseSkills.modules, '48', '212'));
  newModule.modules.push(switchId(baseSkills.modules, '74', '186'));
  newModule.modules.push(switchId(baseSkills.modules, '77', '183'));
  db.collection('categoriesformations').findOneAndUpdate(
    { id: 'baseSkills' },
    { $set: { modules: newModule.modules } }
  );

  // Access Right
  newModule = { modules: [] };
  const accessRight: ApticDoc = await db.collection('categoriesformations').findOne({ id: 'accessRight' });
  newModule.modules.push(switchId(accessRight.modules, '84', '176'));
  newModule.modules.push(switchId(accessRight.modules, '85', '175'));
  newModule.modules.push(switchId(accessRight.modules, '86', '174'));
  newModule.modules.push(switchId(accessRight.modules, '87', '173'));
  newModule.modules.push(switchId(accessRight.modules, '88', '172'));
  newModule.modules.push(switchId(accessRight.modules, '89', '171'));
  newModule.modules.push(switchId(accessRight.modules, '93', '167'));
  newModule.modules.push(switchId(accessRight.modules, '95', '165'));
  db.collection('categoriesformations').findOneAndUpdate(
    { id: 'accessRight' },
    { $set: { modules: newModule.modules } }
  );

  // Parenting
  newModule = { modules: [] };
  const parentingHelp: ApticDoc = await db.collection('categoriesformations').findOne({ id: 'parentingHelp' });
  newModule.modules.push(switchId(parentingHelp.modules, '3', '257'));
  newModule.modules.push(switchId(parentingHelp.modules, '22', '238'));
  newModule.modules.push(switchId(parentingHelp.modules, '82', '178'));
  newModule.modules.push(switchId(parentingHelp.modules, '94', '166'));
  db.collection('categoriesformations').findOneAndUpdate(
    { id: 'parentingHelp' },
    { $set: { modules: newModule.modules } }
  );

  // Social and professional
  newModule = { modules: [] };
  const socialAndProfessional: ApticDoc = await db
    .collection('categoriesformations')
    .findOne({ id: 'socialAndProfessional' });
  newModule.modules.push(switchId(socialAndProfessional.modules, '6', '254'));
  newModule.modules.push(switchId(socialAndProfessional.modules, '20', '240'));
  newModule.modules.push(switchId(socialAndProfessional.modules, '66', '194'));
  newModule.modules.push(switchId(socialAndProfessional.modules, '67', '193'));
  newModule.modules.push(switchId(socialAndProfessional.modules, '68', '192'));
  newModule.modules.push(switchId(socialAndProfessional.modules, '69', '191'));
  newModule.modules.push(switchId(socialAndProfessional.modules, '124', '262'));
  newModule.modules.push(switchId(socialAndProfessional.modules, '125', '263'));
  newModule.modules.push(switchId(socialAndProfessional.modules, '127', '003'));
  db.collection('categoriesformations').findOneAndUpdate(
    { id: 'socialAndProfessional' },
    { $set: { modules: newModule.modules } }
  );

  // Digital security
  newModule = { modules: [] };
  const digitalCultureSecurity: ApticDoc = await db
    .collection('categoriesformations')
    .findOne({ id: 'digitalCultureSecurity' });
  newModule.modules.push(switchId(digitalCultureSecurity.modules, '2', '264'));
  newModule.modules.push(switchId(digitalCultureSecurity.modules, '5', '255'));
  newModule.modules.push(switchId(digitalCultureSecurity.modules, '9', '265'));
  newModule.modules.push(switchId(digitalCultureSecurity.modules, '28', '232'));
  newModule.modules.push(switchId(digitalCultureSecurity.modules, '34', '225'));
  newModule.modules.push(switchId(digitalCultureSecurity.modules, '39', '221'));
  newModule.modules.push(switchId(digitalCultureSecurity.modules, '42', '218'));
  newModule.modules.push(switchId(digitalCultureSecurity.modules, '51', '209'));
  newModule.modules.push(switchId(digitalCultureSecurity.modules, '52', '208'));
  newModule.modules.push(switchId(digitalCultureSecurity.modules, '54', '206'));
  newModule.modules.push(switchId(digitalCultureSecurity.modules, '65', '195'));
  newModule.modules.push(switchId(digitalCultureSecurity.modules, '96', '164'));
  newModule.modules.push(switchId(digitalCultureSecurity.modules, '97', '163'));
  newModule.modules.push(switchId(digitalCultureSecurity.modules, '98', '162'));
  db.collection('categoriesformations').findOneAndUpdate(
    { id: 'digitalCultureSecurity' },
    { $set: { modules: newModule.modules } }
  );
}

function switchStructureId(moduleArray: [], originalId: string, newId: string): string {
  const module = moduleArray.find((id) => id === originalId);
  if (module) {
    return newId;
  }
  return null;
}

function switchId(moduleArray: ApticModule[], originalId: string, newId: string): ApticModule {
  const module = moduleArray.find((skill) => skill.id === originalId);
  module.id = newId;
  module.display_id = newId;
  module.display_name = `Modules APTIC - n°${newId}`;
  delete module.receipt_time;
  delete module.last_update_time;
  delete module.url;
  return module;
}
