import { getModelToken } from '@nestjs/mongoose';
import { Test, TestingModule } from '@nestjs/testing';
import { ParametersController } from './parameters.controller';
import { ParametersService } from './parameters.service';
import { Parameters } from './schemas/parameters.schema';

describe('ParametersController', () => {
  let controller: ParametersController;

  const parametersServiceMock = {
    getParameters: jest.fn(),
    setParameterLockdownInfoDisplay: jest.fn(),
  };

  beforeEach(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: ParametersService,
          useValue: parametersServiceMock,
        },
        {
          provide: getModelToken('Parameters'),
          useValue: Parameters,
        },
      ],
      controllers: [ParametersController],
    }).compile();

    controller = module.get<ParametersController>(ParametersController);
  });

  it('should be defined', async () => {
    expect(controller).toBeDefined();
  });

  it('should call getParameters', async () => {
    const spyer = jest.spyOn(parametersServiceMock, 'getParameters');
    await controller.getParameters();
    expect(spyer).toBeCalledTimes(1);
  });

  describe('setParameterLockdownInfoDisplay', () => {
    const spyer = jest.spyOn(parametersServiceMock, 'setParameterLockdownInfoDisplay');
    afterEach(() => spyer.mockClear());
    it('should call setParameterLockdownInfoDisplay(false) ', async () => {
      await controller.setParameterLockdownInfoDisplay({ lockdownInfoDisplay: false });
      expect(spyer).toBeCalledTimes(1);
      expect(spyer).toBeCalledWith(false);
    });

    it('should call setParameterLockdownInfoDisplay(true)', async () => {
      await controller.setParameterLockdownInfoDisplay({ lockdownInfoDisplay: true });
      expect(spyer).toBeCalledTimes(1);
      expect(spyer).toBeCalledWith(true);
    });
  });
});
