import { Prop, Schema, SchemaFactory } from '@nestjs/mongoose';
import { Type } from 'class-transformer';
import { IsNotEmpty, ValidateNested } from 'class-validator';
import { Document, SchemaTypes, Types } from 'mongoose';
import { PersonalOfferDocument } from '../../personal-offers/schemas/personal-offer.schema';
import { Week } from '../../shared/schemas/week.schema';
import { StructureTypeDocument } from '../structure-type/structure-type.schema';
import { Address } from './address.schema';
import { StructureCategories } from './structureCategories.schema';

export type StructureDocument = Structure & Document;

@Schema({ timestamps: { createdAt: 'createdAt', updatedAt: 'updatedAt' } })
export class Structure {
  constructor(data: Partial<Structure> = {}) {
    this.structureName = data.structureName;
    this.numero = data.numero;
    this.structureType = data.structureType;
    this.description = data.description;
    this.lockdownActivity = data.lockdownActivity;
    this.address = data.address;
    this.contactPhone = data.contactPhone;
    this.contactMail = data.contactMail;
    this.website = data.website;
    this.facebook = data.facebook;
    this.twitter = data.twitter;
    this.instagram = data.instagram;
    this.linkedin = data.linkedin;
    this.pmrAccess = data.pmrAccess;
    this.otherDescription = data.otherDescription;
    this.exceptionalClosures = data.exceptionalClosures;
    this.remoteAccompaniment = data.remoteAccompaniment;
    this.categories = data.categories;
    this.exceptionalClosures = data.exceptionalClosures;
    this.freeWorkShop = data.freeWorkShop;
    this.nbPrinters = data.nbPrinters;
    this.nbComputers = data.nbComputers;
    this.nbScanners = data.nbScanners;
    this.hours = data.hours;
    this.coord = data.coord;
    this.deletedAt = data.deletedAt;
    this.accountVerified = data.accountVerified;
    this.dataShareConsentDate = data.dataShareConsentDate;
    this.personalOffers = data.personalOffers;
    this.hasNoUserDN = data.hasNoUserDN;
    this.hasUserWithAppointmentDN = data.hasUserWithAppointmentDN;
  }

  @Prop()
  numero: string;

  @Prop()
  @IsNotEmpty()
  structureName: string;

  @Prop({ type: SchemaTypes.ObjectId, ref: 'StructureType' })
  @IsNotEmpty()
  structureType: StructureTypeDocument;

  @Prop()
  description: string;

  @Prop()
  lockdownActivity: string;

  @Prop()
  @ValidateNested({ each: true })
  @Type(() => Address)
  address: Address;

  @Prop()
  contactPhone?: string;

  @Prop()
  contactMail?: string;

  @Prop()
  website: string;

  @Prop()
  facebook: string;

  @Prop()
  twitter: string;

  @Prop()
  instagram: string;

  @Prop()
  linkedin: string;

  @Prop()
  @IsNotEmpty()
  pmrAccess: boolean;

  @Prop()
  otherDescription: string;

  @Prop()
  exceptionalClosures: string;

  @Prop()
  @IsNotEmpty()
  remoteAccompaniment: boolean;

  @Prop({ type: StructureCategories })
  categories: StructureCategories;

  @Prop()
  /** 'Oui' | 'Oui sous condition' | 'Non' */
  freeWorkShop: string;

  @Prop()
  nbComputers: number;

  @Prop()
  @IsNotEmpty()
  nbPrinters: number;

  @Prop()
  @IsNotEmpty()
  nbScanners: number;

  @Prop()
  hours: Week;

  @Prop()
  coord: number[];

  @Prop()
  toBeDeletedAt?: Date;

  @Prop()
  deletedAt: Date;

  @Prop()
  accountVerified: boolean;

  @Prop()
  dataShareConsentDate: Date;

  @Prop({ type: [{ type: Types.ObjectId, ref: 'PersonalOffer' }] })
  personalOffers: PersonalOfferDocument[];

  @Prop()
  createdAt: Date;

  @Prop()
  updatedAt: Date;

  @Prop()
  idCNFS?: string;

  @Prop()
  hasNoUserDN?: boolean;

  @Prop()
  hasUserWithAppointmentDN?: boolean;

  // No @Prop decorator because this property must not be stored in database
  categoriesWithPersonalOffers: StructureCategories;
}

export const StructureSchema = SchemaFactory.createForClass(Structure);
StructureSchema.index({ '$**': 'text' });
