import { Test, TestingModule } from '@nestjs/testing';
import { getModelToken } from '@nestjs/mongoose';
import { StructuresImportService } from './structures-import.service';
import { of } from 'rxjs';
import { DataGouvStructure } from '../interfaces/data-gouv-structure.interface';
import { HttpService } from '@nestjs/axios';
import { AxiosResponse } from 'axios';
import { mockGouvStructure, mockGouvStructureToResinFormat } from '../../../test/mock/data/gouvStructures.mock.data';

describe('StructuresImportService', () => {
  let service: StructuresImportService;
  let httpService: HttpService;

  beforeEach(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        StructuresImportService,
        {
          provide: HttpService,
          useValue: {
            get: jest.fn(),
          },
        },
        {
          provide: getModelToken('Structure'),
          useValue: {
            countDocuments: jest.fn(),
            new: jest.fn(),
            save: jest.fn(),
          },
        },
      ],
    }).compile();

    service = module.get<StructuresImportService>(StructuresImportService);
    httpService = module.get<HttpService>(HttpService);
  });

  describe('importDataGouvStructures', () => {
    it('should create a new structure when doesAlreadyExist returns false', async () => {
      const mockedAxiosResponse: AxiosResponse<DataGouvStructure[]> = {
        data: [mockGouvStructure],
        status: 200,
        statusText: 'OK',
        headers: {},
        config: {},
      };
      const saveSpy = jest.fn();
      const mockStructureModel = jest.fn().mockImplementation(() => ({ save: saveSpy }));
      (service as any).structureModel = mockStructureModel;

      const formatToResinSchemaSpy = jest.spyOn(service as any, 'formatToResinSchema').mockResolvedValueOnce('called');
      const doesAlreadyExistSpy = jest.spyOn(service as any, 'doesAlreadyExist').mockResolvedValueOnce(false);
      jest.spyOn(httpService, 'get').mockImplementationOnce(() => of(mockedAxiosResponse));

      await service.importDataGouvStructures();

      expect(httpService.get).toBeCalled();
      expect(formatToResinSchemaSpy).toBeCalled();
      expect(doesAlreadyExistSpy).toBeCalled();
      expect(saveSpy).toBeCalled();
    });
  });

  describe('formatToResinSchema', () => {
    it('should transform data.gouv structures to resin format', async () => {
      const result = await service['formatToResinSchema'](mockGouvStructure);
      expect(result).toEqual(mockGouvStructureToResinFormat);
    });
  });
});
