import { HttpModule } from '@nestjs/axios';
import { HttpStatus } from '@nestjs/common';
import { getModelToken } from '@nestjs/mongoose';
import { Test, TestingModule } from '@nestjs/testing';
import { Types } from 'mongoose';
import { ConfigurationModule } from '../../configuration/configuration.module';
import { MailerService } from '../../mailer/mailer.service';
import { CreateJobsGroupsDto } from '../dto/create-jobsGroups.dto';
import { JobsGroupsService } from './jobsGroups.service';
import { UsersService } from './users.service';
import { JobsService } from './jobs.service';

describe('JobsGroupsService', () => {
  let service: JobsGroupsService;

  const mockJobsGroupsModel = {
    create: jest.fn(),
    find: jest.fn().mockReturnThis(),
    collation: jest.fn().mockReturnThis(),
    sort: jest.fn().mockReturnThis(),
    exec: jest.fn(),
    findOne: jest.fn().mockReturnThis(),
    findById: jest.fn().mockReturnThis(),
    deleteOne: jest.fn().mockReturnThis(),
  };

  const mockJobModel = {
    create: jest.fn(),
    find: jest.fn().mockReturnThis(),
    populate: jest.fn().mockReturnThis(),
    collation: jest.fn().mockReturnThis(),
    sort: jest.fn().mockReturnThis(),
    exec: jest.fn().mockReturnThis(),
    findOne: jest.fn(),
    findById: jest.fn(),
    deleteOne: jest.fn(),
  };

  const mockUserModel = {
    find: jest.fn(),
    findOne: jest.fn(),
    findById: jest.fn(),
  };
  const mockUserService = {
    replaceJobsGroups: jest.fn(),
    getAdmins: jest.fn(),
  };

  beforeEach(async () => {
    const module: TestingModule = await Test.createTestingModule({
      imports: [ConfigurationModule, HttpModule],
      providers: [
        JobsGroupsService,
        {
          provide: getModelToken('JobsGroups'),
          useValue: mockJobsGroupsModel,
        },
        JobsService,
        {
          provide: getModelToken('Job'),
          useValue: mockJobModel,
        },
        {
          provide: UsersService,
          useValue: mockUserService,
        },
        {
          provide: getModelToken('User'),
          useValue: mockUserModel,
        },
        MailerService,
      ],
    }).compile();
    service = module.get<JobsGroupsService>(JobsGroupsService);
  });

  it('should be defined', () => {
    expect(service).toBeDefined();
  });

  describe('findAll', () => {
    it('should findAll all jobs groups', async () => {
      const result = [
        {
          _id: new Types.ObjectId('6231aefe76598527c8d0b5bc'),
          name: 'Technique',
        },
      ];
      mockJobsGroupsModel.exec.mockResolvedValueOnce(result);
      const reply = await service.findAll(false);
      expect(reply.length).toBe(1);
    });
  });

  it('findByName', async () => {
    mockJobsGroupsModel.exec.mockResolvedValueOnce({
      _id: new Types.ObjectId('6231aefe76598527c8d0b5bc'),
      name: 'Technique',
    });
    const reply = await service.findByName('Technique');
    expect(reply).toBeTruthy();
  });
  it('findOne', async () => {
    mockJobsGroupsModel.exec.mockResolvedValueOnce({
      _id: new Types.ObjectId('6231aefe76598527c8d0b5bc'),
      name: 'Technique',
    });
    const reply = await service.findOne('6231aefe76598527c8d0b5bc');
    expect(reply).toBeTruthy();
  });

  describe('createJobsGroups', () => {
    it('create', async () => {
      mockJobsGroupsModel.create.mockResolvedValueOnce({
        _id: new Types.ObjectId('6231aefe76598527c8d0b5bc'),
        name: 'Technique',
      });
      const createJobsGroups: CreateJobsGroupsDto = {
        name: 'Technique',
      };
      const reply = await service.create(createJobsGroups);
      expect(reply).toBeTruthy();
    });
  });

  describe('update', () => {
    it('should update', async () => {
      const newJobsGroupsDto = {
        name: 'Technique',
      };
      const newJobsGroups = {
        _id: new Types.ObjectId('6231aefe76598527c8d0b5bc'),
        name: 'Technique',
      };
      mockJobsGroupsModel.exec.mockResolvedValueOnce({
        _id: new Types.ObjectId('6231aefe76598527c8d0b5bc'),
        name: 'Technique old',
        save: jest.fn().mockResolvedValueOnce(newJobsGroups),
      });
      const reply = await service.update('623aed68c5d45b6fbbaa7e60', newJobsGroupsDto);
      expect(reply.name).toBe('Technique');
    });
  });

  it('should throw error if not found', async () => {
    const newJobsGroupsDto = {
      name: 'Technique',
    };
    mockJobsGroupsModel.exec.mockResolvedValueOnce(null);
    try {
      await service.update('623aed68c5d45b6fbbaa7e60', newJobsGroupsDto);
      expect(true).toBe(false);
    } catch (e) {
      expect(e.status).toBe(HttpStatus.NOT_FOUND);
    }
  });
});
